@@ -3,7 +3,6 @@
 from __future__ import annotations
 
 import copy
-import json
 import re
 from abc import ABC
 from abc import abstractmethod
@@ -25,12 +24,13 @@ from .function_extensions import FilterFunction
 from .match import NodeList
 from .selectors import Filter as FilterSelector
 from .selectors import ListSelector
+from .serialize import canonical_string
 
 if TYPE_CHECKING:
     from .path import JSONPath
     from .selectors import FilterContext
 
-# ruff: noqa: D102
+# ruff: noqa: D102, PLW1641
 
 
 class FilterExpression(ABC):
@@ -208,7 +208,7 @@ class StringLiteral(Literal[str]):
     __slots__ = ()
 
     def __str__(self) -> str:
-        return json.dumps(self.value)
+        return canonical_string(self.value)
 
 
 class IntegerLiteral(Literal[int]):
@@ -243,8 +243,7 @@ class RegexLiteral(Literal[Pattern[str]]):
             if self.value.flags & flag:
                 flags.append(ch)
 
-        pattern = re.sub(r"\\(.)", r"\1", self.value.pattern)
-        return f"/{pattern}/{''.join(flags)}"
+        return f"/{self.value.pattern}/{''.join(flags)}"
 
 
 class ListLiteral(FilterExpression):
@@ -318,7 +317,7 @@ class PrefixExpression(FilterExpression):
 class InfixExpression(FilterExpression):
     """A pair of expressions and a comparison or logical operator."""
 
-    __slots__ = ("left", "operator", "right")
+    __slots__ = ("left", "operator", "right", "logical")
 
     def __init__(
         self,
@@ -329,10 +328,11 @@ class InfixExpression(FilterExpression):
         self.left = left
         self.operator = operator
         self.right = right
+        self.logical = operator in ("&&", "||")
         super().__init__()
 
     def __str__(self) -> str:
-        if self.operator in ("&&", "||"):
+        if self.logical:
             return f"({self.left} {self.operator} {self.right})"
         return f"{self.left} {self.operator} {self.right}"
 
@@ -346,22 +346,22 @@ class InfixExpression(FilterExpression):
 
     def evaluate(self, context: FilterContext) -> bool:
         left = self.left.evaluate(context)
-        if isinstance(left, NodeList) and len(left) == 1:
+        if not self.logical and isinstance(left, NodeList) and len(left) == 1:
             left = left[0].obj
 
         right = self.right.evaluate(context)
-        if isinstance(right, NodeList) and len(right) == 1:
+        if not self.logical and isinstance(right, NodeList) and len(right) == 1:
             right = right[0].obj
 
         return context.env.compare(left, self.operator, right)
 
     async def evaluate_async(self, context: FilterContext) -> bool:
         left = await self.left.evaluate_async(context)
-        if isinstance(left, NodeList) and len(left) == 1:
+        if not self.logical and isinstance(left, NodeList) and len(left) == 1:
             left = left[0].obj
 
         right = await self.right.evaluate_async(context)
-        if isinstance(right, NodeList) and len(right) == 1:
+        if not self.logical and isinstance(right, NodeList) and len(right) == 1:
             right = right[0].obj
 
         return context.env.compare(left, self.operator, right)
@@ -375,6 +375,12 @@ class InfixExpression(FilterExpression):
         self.right = children[1]
 
 
+PRECEDENCE_LOWEST = 1
+PRECEDENCE_LOGICAL_OR = 3
+PRECEDENCE_LOGICAL_AND = 4
+PRECEDENCE_PREFIX = 7
+
+
 class BooleanExpression(FilterExpression):
     """An expression that always evaluates to `True` or `False`."""
 
@@ -408,13 +414,40 @@ class BooleanExpression(FilterExpression):
         )
 
     def __str__(self) -> str:
-        return str(self.expression)
+        return self._canonical_string(self.expression, PRECEDENCE_LOWEST)
 
     def __eq__(self, other: object) -> bool:
         return (
             isinstance(other, BooleanExpression) and self.expression == other.expression
         )
 
+    def _canonical_string(
+        self, expression: FilterExpression, parent_precedence: int
+    ) -> str:
+        if isinstance(expression, InfixExpression):
+            if expression.operator == "&&":
+                left = self._canonical_string(expression.left, PRECEDENCE_LOGICAL_AND)
+                right = self._canonical_string(expression.right, PRECEDENCE_LOGICAL_AND)
+                expr = f"{left} && {right}"
+                return (
+                    f"({expr})" if parent_precedence >= PRECEDENCE_LOGICAL_AND else expr
+                )
+
+            if expression.operator == "||":
+                left = self._canonical_string(expression.left, PRECEDENCE_LOGICAL_OR)
+                right = self._canonical_string(expression.right, PRECEDENCE_LOGICAL_OR)
+                expr = f"{left} || {right}"
+                return (
+                    f"({expr})" if parent_precedence >= PRECEDENCE_LOGICAL_OR else expr
+                )
+
+        if isinstance(expression, PrefixExpression):
+            operand = self._canonical_string(expression.right, PRECEDENCE_PREFIX)
+            expr = f"!{operand}"
+            return f"({expr})" if parent_precedence > PRECEDENCE_PREFIX else expr
+
+        return str(expression)
+
     def evaluate(self, context: FilterContext) -> bool:
         return context.env.is_truthy(self.expression.evaluate(context))
 
@@ -667,6 +700,9 @@ class CurrentKey(FilterExpression):
         super().__init__()
         self.volatile = True
 
+    def __str__(self) -> str:
+        return "#"
+
     def __eq__(self, other: object) -> bool:
         return isinstance(other, CurrentKey)
 
