@@ -8,17 +8,14 @@ TOKEN_ILLEGAL = sys.intern("ILLEGAL")
 TOKEN_SKIP = sys.intern("SKIP")
 
 # JSONPath expression tokens
-TOKEN_BRACKET_PROPERTY = sys.intern("BRACKET_PROPERTY")
 TOKEN_COLON = sys.intern("COLON")
 TOKEN_COMMA = sys.intern("COMMA")
 TOKEN_DDOT = sys.intern("DDOT")
 TOKEN_DOT = sys.intern("DOT")
 TOKEN_DOT_INDEX = sys.intern("DINDEX")
 TOKEN_DOT_PROPERTY = sys.intern("DOT_PROPERTY")
-TOKEN_FILTER_END = sys.intern("FILTER_END")
-TOKEN_FILTER_START = sys.intern("FILTER_START")
-TOKEN_IDENT = sys.intern("IDENT")
-TOKEN_INDEX = sys.intern("IDX")
+TOKEN_FILTER = sys.intern("FILTER")
+TOKEN_FAKE_ROOT = sys.intern("FAKE_ROOT")
 TOKEN_KEY = sys.intern("KEY")
 TOKEN_KEYS = sys.intern("KEYS")
 TOKEN_RBRACKET = sys.intern("RBRACKET")
@@ -26,10 +23,7 @@ TOKEN_BARE_PROPERTY = sys.intern("BARE_PROPERTY")
 TOKEN_LIST_SLICE = sys.intern("LSLICE")
 TOKEN_LIST_START = sys.intern("LBRACKET")
 TOKEN_PROPERTY = sys.intern("PROP")
-TOKEN_QUESTION = sys.intern("QUESTION")
-TOKEN_QUOTE_PROPERTY = sys.intern("QUOTE_PROPERTY")
 TOKEN_ROOT = sys.intern("ROOT")
-TOKEN_SLICE = sys.intern("SLICE")
 TOKEN_SLICE_START = sys.intern("SLICE_START")
 TOKEN_SLICE_STEP = sys.intern("SLICE_STEP")
 TOKEN_SLICE_STOP = sys.intern("SLICE_STOP")
@@ -118,8 +112,11 @@ class Token:
             and self.path == other.path
         )
 
+    def __hash__(self) -> int:
+        return hash((self.kind, self.value, self.index, self.path))
+
     def position(self) -> Tuple[int, int]:
         """Return the line and column number for the start of this token."""
         line_number = self.value.count("\n", 0, self.index) + 1
         column_number = self.index - self.value.rfind("\n", 0, self.index)
-        return (line_number, column_number)
+        return (line_number, column_number - 1)
